<?php
/**
 * VFM Core class
 * 
 * @version $Id: VFM_Core.class.php 11 2006-05-21 04:21:30Z tvlgiao $
 * @author Giao Trinh (tvlgiao@vnetcom.com)
 * @package Joomla
 * @subpackage VFM
 * @copyright (C) 2006 VNETCOM Corp.
 * @license GNU/LGPL
 */

/** requires */
require_once dirname(__FILE__) . '/ZipFile.class.php';

/**
 * VFM Core Class
 *
 * @package Joomla
 * @subpackage VFM
 */
class VFM_Core {

	############################################################################
	############################################################################
	# VARIABLES

	/**
	 * hold configuration
	 */
	var $conf = array();
	
	/**
	 * current working Directory or File
	 */
	var $dirname;
	
	/**
	 * specify current instance for file or directory
	 */
	var $isFolder;
	
	/**
	 * working Dir/File's Permission (admin, view, deny)
	 */
	var $permission;
	
	/**
	 * object created by dir() for current directory
	 */
	var $oDir;
	
	/**
	 * hold sub-directory/file entry current walking of this directory
	 */
	var $entry = array();
	
	/**
	 * records of children stored in database
	 */
	var $records = array();
	
	/**
	 * hash records indexed by Filename
	 */
	var $recordsByFilename = array();
	
	/**
	 * last error message
	 */
	var $errorMsg;
	
	/**
	 * cache all sub entries of this directory
	 * @access private
	 */
	var $_entries = array();
	
	/**
	 * walking index of current sub entry
	 * @access private
	 */
	var $_entries_i = 0;
	
	/**
	 * hold all group names and their permissions of all this directory's children
	 * which are loaded in database if working directory is 'admin'
	 */
	var $filesgroups = array('usertypes' => array(), 'groups' => array());
	
	/**
	 * list of all VFM groups and Joomla usertypes
	 */
	var $groups = array('usertypes' => array(), 'groups' => array());
	
	/**
	 * hash list of all groups indexed by their Name
	 */
	var $_groupsByName = array('usertypes' => array(), 'groups' => array());
	
	var $_debug_count = 0;
	
	var $_recursed = false;


	############################################################################
	############################################################################
	# STATIC METHODS
	

	/**
	 * modify input directory/file from REQUEST make it safe
	 *
	 * @param string $dir example $_REQUEST['dir']
	 * @return string safe name
	 */
	/*static*/ function trimPath($dir) {
		$dir = trim(preg_replace('/\.\./', '', $dir));
		$dir = preg_replace('/(^[\\\\\/]+)|([\\\\\/]+$)/', '', $dir);
		return $dir;
	}
	
	/**
	 * return absolute dir/file name
	 * 
	 * @param string $rootdir absolute root directory
	 * @param string $path relative current directory/file
	 * @param boolean $trim if true method call trimPath to make sure input $path safe
	 * @return string absolute name
	 */
	/*static*/ function realPath($rootdir, $path, $trim = true) {
		if ($trim) $path = VFM_Core::trimPath($path);
		return str_replace('\\', '/', realpath ($rootdir . '/' . $path));
	}
	
	/**
	 * check relative $child name is child of relative $parent name
	 *
	 * @param string $child relative sub dir/file name
	 * @param string $parent relative parent dir/file name
	 * @return boolean
	 */
	/*static*/ function isChild($child, $parent) {
		if ($parent === '') return true;
		return strpos($child, $parent . '/') === 0 ? true : false;
	}
	
	/**
	 * get parent of input dir/file name
	 *
	 * @param string $dir input dir/file name
	 * @return string parent directory name
	 */
	/*static*/ function getParent($dir) {
		return preg_replace ('/(\\/[^\\/]*$)|(^[^\\/]+$)/', '', $dir);
	}
	
	/*static*/ function comparePermission() {
		$prior = array ('admin' => 3, 'deny' => 2, 'view' => 1);
		$l = 0;
		
		$numargs = func_num_args();
		$arg_list = func_get_args();
		for ($i = 0; $i < $numargs; $i++) {
			$l = max ($l, intval($prior[$arg_list[$i]]));
		}
			
		return array_search ($l, $prior);
	}
	
	/*static*/ function getUsertype($usertype) {
		$usertype = str_replace(' ', '', strtolower($usertype));
		if ($usertype == "registered") return "user";
		if ($usertype == "") return "guest";
		else return $usertype;
	}
	
	############################################################################
	############################################################################
	# METHODS
	
	/**
	 * constructor
	 *
	 * @param array $conf array of configuration
	 */
	function VFM_Core($conf) {
		global $my;
		$this->conf = $conf;
		$this->_debug(&$_REQUEST, '$_REQUEST', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
		$this->_debug(&$my, '$my', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
		$this->loadGroups();
	}
	
	/**
	 * load groups, usertypes in database
	 */
	function loadGroups() {
		global $database;
		
		# usertypes
		$database->setQuery("SELECT * FROM #__usertypes");
		$this->groups['usertypes'] = $database->loadObjectList();
		foreach ($this->groups['usertypes'] as $k => $r)
			$this->groupsByName['usertypes'][$r->name] = &$this->groups['usertypes'][$k];
		
		# VFM groups
		$database->setQuery("SELECT * FROM #__vfm_groups");
		$this->groups['groups'] = $database->loadObjectList();
		foreach ($this->groups['groups'] as $k => $r)
			$this->groupsByName['groups'][$r->gid] = &$this->groups['groups'][$k];
		
		$this->_debug(&$this->groups, '$this->groups', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
		$this->_debug(&$this->groupsByName, '$this->groupsByName', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
	}
	
	
	/**
	 * manually set absolute root directory $conf['rootpath']
	 *
	 * @param string $root
	 */
	function setRoot($root) {
		$this->conf['rootdir'] = realpath($root);
	}
	
	/**
	 * set current working directory, also initilize this
	 *
	 * @param string $dir directory from $_REQUEST;
	 */
	function setDir($dir) {
		$this->dirname = VFM_Core::trimPath($dir);
		$this->isFolder = is_dir($this->conf['rootdir'] . '/' . $this->dirname);
		
		$this->initDir();
	}
	
	/**
	 * initilize working directory
	 *
	 * load its permission and its sub dirs/files's permissions into $records.
	 * - if has admin permission, load full groups, permissions of sub dirs/files.
	 * - sub dirs/files has admin permission will load all theirs groups.
	 *
	 * @see $records
	 */
	function initDir($recurse = false) {
		global $database, $my;
		
		$this->_recursed = $recurse;
		
		$usertype = VFM_Core::getUsertype($my->usertype);
		
		#
		# load parents, current, sub
		#
		#$query = "
		#	SELECT f.*, fu.permission FROM #__vfm_link_files_usertypes as fu
		#	INNER JOIN #__vfm_files as f ON fu.file_id=f.id
		#	INNER JOIN #__usertypes as u ON fu.usertype_id=u.id
		#	WHERE 
		#		( 	f.filename LIKE '$this->dirname' OR 
		#			INSTR('$this->dirname', CONCAT(f.filename, '/')) = 1 OR 
		#			f.parent LIKE '$this->dirname' ) AND 
		#		u.name LIKE '$usertype'
		#	ORDER BY f.isfolder DESC, f.filename
		#	";


		#
		# user's usertype ID
		#
		$database->setQuery("SELECT id FROM #__usertypes WHERE name LIKE " . $database->Quote($usertype));
		$database->loadObject(&$r);
		$uid = $r->id;
		$this->myUsertypeId = $uid;
		$this->_debug(&$database, '$database', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
		$this->_debug(&$this->myUsertypeId, '$this->myUsertypeId', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
		

		#
		# groups that user is in
		#
		$query = "
			SELECT g.gid FROM #__vfm_groups as g
			INNER JOIN #__vfm_link_users_groups as ug ON g.gid=ug.gid
			WHERE ug.user_id='$my->id'
			";
		$database->setQuery($query);
		$rows = $database->loadObjectList();
		$this->myGroups = $rows;
		$INgid = array();
		foreach ($rows as $r) $INgid[] = $r->gid;
		$INgid = "('" . implode("','", $INgid) . "')";
		$this->_debug(&$database, '$database', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
		$this->_debug(&$this->myGroups, '$this->myGroups', __LINE__, __FUNCTION__, __CLASS__, __FILE__);

		
		#
		# load current directory/file, parent and children
		#		
		$_parent = $recurse 
					? "f.filename LIKE " . $database->Quote($this->dirname ? "$this->dirname./%" : "%")
					: "f.parent LIKE " . $database->Quote($this->dirname);
		$query = "
			SELECT f.*, u.permission as usertype_permission, g.permission as group_permission
			FROM #__vfm_files AS f
			LEFT JOIN #__vfm_link_files_usertypes AS u ON (f.id=u.file_id AND u.usertype_id='$uid')
			LEFT JOIN #__vfm_link_files_groups AS g ON (f.id=g.file_id AND g.gid IN $INgid)
			WHERE
				( f.filename LIKE ".$database->Quote($this->dirname)." OR 
				INSTR(".$database->Quote($this->dirname).", CONCAT(f.filename, '/')) = 1 OR 
				  $_parent ) "./*AND 
				( g.gid IS NOT NULL OR u.usertype_id IS NOT NULL )*/"
				
			ORDER BY f.isfolder DESC, f.filename
			";
		$database->setQuery($query);
		$this->records = $database->loadObjectList();
		$this->_debug(&$database, '$database', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
		$this->_debug(&$this->records, '$this->records', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
		
		#
		# records index by filename, sorted
		#
		$this->recordsByFilename = array();
		foreach ($this->records as $k => $row) {
			if (!isset($this->recordsByFilename[$row->filename])) {
				$this->recordsByFilename[$row->filename] = &$this->records[$k];
			}
			# choose permission...
			$this->recordsByFilename[$row->filename]->permission = 
				VFM_Core::comparePermission($this->recordsByFilename[$row->filename]->permission,
											$row->usertype_permission, 
											$row->group_permission );
		}
		ksort($this->recordsByFilename, SORT_STRING);
		$this->_debug(&$this->recordsByFilename, '$this->recordsByFilename', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
		
		#
		# set permission of this directory
		#
		$this->permission = $this->conf['root_permission'];
		$this->permission = $this->getPermission($this->dirname);
		if ($this->permission == 'admin')
			$this->loadFullPermissions();
		elseif ($this->permission == 'view')
			$this->loadPartialPermissions();
		
		$this->_debug(&$this, '$this', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
	}
	
	/**
	 * load all groups of sub dirs/files which user is 'admin'
	 */
	function loadPartialPermissions() {
		global $database;
		
		$INgid = array();
		foreach ($this->myGroups as $r) $INgid[] = $r->gid;
		$INgid = "('" . implode("','", $INgid) . "')";
		
		#
		# file ID which i'm Admin
		#
		$_parent = $this->_recursed
					? "f.filename LIKE " . $database->Quote($this->dirname ? "$this->dirname/%" : "%")
					: "f.parent LIKE " . $database->Quote($this->dirname);
		$query = "
			SELECT f.id 
			FROM #__vfm_files AS f 
			LEFT JOIN #__vfm_link_files_usertypes AS fu ON f.id=fu.file_id
			LEFT JOIN #__vfm_link_files_groups AS fg ON f.id=fg.file_id
			WHERE
				$_parent AND
				( fu.permission = 'admin' OR fg.permission = 'admin' ) AND
				( fu.usertype_id = '$this->myUsertypeId' OR fg.gid IN $INgid )
			";
		$database->setQuery($query);
		$rows = $database->loadObjectList();
		$INid = array();
		foreach ($rows as $r) $INid[] = $r->id;
		$INid = "('" . implode("','", $INid) . "')";
		
		$this->_debug(&$database, '$database', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
		$this->_debug(&$rows, '$rows', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
				
					
		# select all groups of children which are managed by me.
		# lay ve tat ca nho'm cua cac folder/file con ma` /me la admin
		
		# usertypes
		$query = "
				SELECT fu.permission, f.*, u.name FROM #__vfm_link_files_usertypes as fu
				INNER JOIN #__vfm_files as f ON fu.file_id=f.id
				INNER JOIN #__usertypes as u ON fu.usertype_id=u.id
				WHERE fu.file_id IN $INid
				";
		$database->setQuery($query);
		$rows = $database->loadObjectList();
		foreach ($rows as $r)
			$this->filesgroups['usertypes'][$r->filename][$r->permission][] = $r->name;
			
		$this->_debug(&$database, '$database', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
		$this->_debug(&$rows, '$rows', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
				
		# groups
		$query = "
				SELECT fg.permission, fg.gid, f.* FROM #__vfm_link_files_groups as fg
				INNER JOIN #__vfm_files as f ON fg.file_id=f.id
				WHERE fg.file_id IN $INid
				";
		$database->setQuery($query);
		$rows = $database->loadObjectList();
		foreach ($rows as $r)
			$this->filesgroups['groups'][$r->filename][$r->permission][] = $r->gid;
			
		$this->_debug(&$database, '$database', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
		$this->_debug(&$rows, '$rows', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
		
		$this->_debug(&$this->filesgroups, '$this->filesgroups', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
	}
	
	/**
	 * get permissions (view, admin) set for all groups of all child files/directories
	 * used for admin manages
	 */
	function loadFullPermissions($recurse = false) {
		global $database, $my;
		
		# usertypes
		$_parent = $this->_recursed 
					? "f.filename LIKE " . $database->Quote($this->dirname ? "$this->dirname/%" : "%")
					: "f.parent LIKE " . $database->Quote($this->dirname);
		$query = "
			SELECT f.*, fu.permission, u.name FROM #__vfm_link_files_usertypes as fu
			INNER JOIN #__vfm_files as f ON fu.file_id=f.id
			INNER JOIN #__usertypes as u ON fu.usertype_id=u.id
			WHERE $_parent
			ORDER BY f.isfolder DESC, f.filename
			";
		$database->setQuery($query);
		$rows = $database->loadObjectList();
		foreach ($rows as $r)
			$this->filesgroups['usertypes'][$r->filename][$r->permission][] = $r->name;
			
		$this->_debug(&$database, '$database', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
		$this->_debug(&$rows, '$rows', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
			
		# groups
		$query = "
			SELECT fg.permission, fg.gid, f.* FROM #__vfm_link_files_groups as fg
			INNER JOIN #__vfm_files as f ON fg.file_id=f.id
			WHERE $_parent
			ORDER BY f.isfolder DESC, f.filename
			";
		$database->setQuery($query);
		$rows = $database->loadObjectList();
		foreach ($rows as $r)
			$this->filesgroups['groups'][$r->filename][$r->permission][] = $r->gid;
			
		$this->_debug(&$database, '$database', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
		$this->_debug(&$rows, '$rows', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
		
		$this->_debug(&$this->filesgroups, '$this->filesgroups', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
	}
	
	/**
	 * get permission of a directory/file
	 *
	 * - permission of sub directories/files are belong to their parent.
	 * - root directory is belong to default configuared permission.
	 * - Super Administrator is always 'admin'
	 *
	 * @param string $filename relative path filename
	 * @return string admin, view or deny
	 */
	function getPermission($filename) {
		global $my;
		if ($my->usertype == 'Super Administrator') return 'admin';
		
		if ($filename === '') return 'view';
		
		# filename is child of `admin`|`denied` working dir
		# return parent's perm
		if (VFM_Core::isChild($filename, $this->dirname) && $this->permission != 'view')
			return $this->permission;
		
		$perm = $this->conf['root_permission'];
		foreach ($this->recordsByFilename as $fn => $row) {
			//if (!$row->permission)
				# parent is root: get root_perms, otherwise parent's perm
				/*$row->permission = VFM_Core::isChild($filename, $this->dirname) && $this->dirname 
														? $this->permission 
														: $this->conf['root_permission'];*/
			# filename is child of record and record is admin|deny
			# return record's perm
			if (VFM_Core::isChild($filename, $fn)) {
				if ($row->permission = 'view') $perm = 'view';
				elseif ($row->permission) return $row->permission;
			}
			
			# filename matches record, return record's perm
			if ($filename == $fn && $row->permission) return $row->permission;
			//if ($row->permission == 'view') $perm = $row->permission;
		}
		return $perm;
	}
	
	/**
	 * set current dir's permission with array of group names
	 *
	 * @param string $perm admin, view or deny
	 * @param array $groups array of group name set to working directory
	 * @param string group names separated by comma (,)
	 */
	function setPermission($perm, $groups) {
		global $database;
		
		if ($this->permission != 'admin') return false;
		
		$database->setQuery("SELECT * FROM #__vfm_files WHERE filename LIKE " . $database->Quote($this->dirname));
		$database->loadObject(&$row);
		$id = $row->id;
		$this->_debug(&$database, '$database', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
		$this->_debug(&$row, '$row', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
		
		if (!$id) {
			$database->setQuery("INSERT INTO #__vfm_files (`filename`, `isfolder`, `parent`) VALUES 
								(" . $database->Quote($this->dirname) .", '" . ($this->isFolder ? 1 : 0) . "', " . $database->Quote(VFM_Core::getParent($this->dirname)) . ")" );
			$database->query();
			$id = $database->insertid();
			$this->_debug(&$database, '$database', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
		}
		
		$result = array();
		$groups_id = $usertypes_id = array();
		foreach ($groups as $name) {
			$name = trim($name);
			if (isset($this->groupsByName['usertypes'][$name]))
				$usertypes_id[] = $this->groupsByName['usertypes'][$name]->id;
			elseif (isset($this->groupsByName['groups'][$name]))
				$groups_id[] = $this->groupsByName['groups'][$name]->gid;
			else continue;
			$result[] = $name;
		}
		
		$this->_debug(&$groups, '$groups', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
		$this->_debug(&$usertypes_id, '$usertypes_id', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
		$this->_debug(&$groups_id, '$groups_id', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
		$this->_debug(&$result, '$result', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
		
		$in = "('" . implode("','", $usertypes_id) . "')";
		$query = "DELETE FROM #__vfm_link_files_usertypes WHERE file_id='$id' AND (permission='$perm' OR usertype_id IN $in)";
		$database->setQuery($query);
		$database->query();
		$this->_debug(&$database, '$database', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
		
		$in = "('" . implode("','", $groups_id) . "')";
		$query = "DELETE FROM #__vfm_link_files_groups WHERE file_id='$id' AND (permission='$perm' OR gid IN $in)";
		$database->setQuery($query);
		$database->query();
		$this->_debug(&$database, '$database', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
		
		$values = array();
		foreach ($usertypes_id as $uid) $values[] = "('$id', '$uid', '$perm')";
		if (count($values) > 0) {
			$query = "INSERT INTO #__vfm_link_files_usertypes (`file_id`, `usertype_id`, `permission`) VALUES " . implode(',', $values);
			$database->setQuery($query);
			$database->query();
			$this->_debug(&$database, '$database', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
		}
		
		$values = array();
		foreach ($groups_id as $gid) $values[] = "('$id', '$gid', '$perm')";
		if (count($values) > 0) {
			$query = "INSERT INTO #__vfm_link_files_groups (`file_id`, `gid`, `permission`) VALUES " . implode(',', $values);
			$database->setQuery($query);
			$database->query();
			$this->_debug(&$database, '$database', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
		}

		
		return implode(', ', $result);
	}
	
	function setAttribute($params) {
		global $database;
		
		if ($this->permission != 'admin') return false;
		
		$set = array();
		foreach ($params as $k => $v) $set[] = "`$k`='" . $database->getEscaped($v) . "'";
		if (!($set = implode (', ' , $set))) return false;
		
		$database->setQuery("SELECT id FROM #__vfm_files WHERE filename LIKE " . $database->Quote($this->dirname));
		$database->loadObject(&$row);
		if (is_object($row)) $id = $row->id;
		unset($row);
		$this->_debug(&$database, '$database', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
		
		if ($id) {
			$database->setQuery("UPDATE #__vfm_files SET $set WHERE id='$id'");
			$_result = $database->query();
			$this->_debug(&$database, '$database', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
			if (!$_result) return false;
		} 
		
		else {
			$title = $params['title'];
			$desc = $params['description'];
			$database->setQuery("INSERT INTO #__vfm_files (`filename`, `isfolder`, `parent`, `title`, `description`) VALUES 
								(" . $database->Quote($this->dirname) . ", '" . ($this->isFolder ? 1 : 0) . "', " . $database->Quote(VFM_Core::getParent($this->dirname)) . ", " . $database->Quote($title) . ", " . $database->Quote($desc) . ")" );
			$_result = $database->query();
			$this->_debug(&$database, '$database', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
			if (!$_result) return false;
		}
		
		$database->setQuery("SELECT * FROM #__vfm_files WHERE filename LIKE " . $database->Quote($this->dirname));
		$database->loadObject(&$row);
		$this->_debug(&$database, '$database', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
		
		return $row ? $row : false;
	}
	
	/**
	 * open current directory for reading their children
	 *
	 * @return boolean result, check $errorMsg
	 * @see $errorMsg
	 */
	function open() {
		global $database;
		
		$this->_entries_i = 0;
		$this->_entries = array();
		
		if ($this->permission == 'deny') {
			$this->errorMsg = 'permission denied';
			return false;
		}
		
		return $this->_readAll();
	}
	
	/**
	 * read all sub dirs/files and cached in $_entries
	 *
	 * @access private
	 * @return boolean result
	 * @see $_entries
	 */
	function _readAll() {
		if (!($d = dir($this->conf['rootdir'] . '/' . $this->dirname))) {
			$this->errorMsg = 'can not open directory';
			return false;
		}
		
		$this->_entries = array();
		while ($entry = $d->read()) {
			$realname = $d->path . '/' . $entry;
			$filename = VFM_Core::trimPath($this->dirname . '/' . $entry);
			$ext = ($ext = strrpos($entry, '.')) !== false ? substr($entry, $ext+1) : '';
			$perm = $this->getPermission($filename);
			$usertypes = array (
				'view' => is_array($this->filesgroups['usertypes'][$filename]['view']) ? $this->filesgroups['usertypes'][$filename]['view'] : array(),
				'admin' => is_array($this->filesgroups['usertypes'][$filename]['admin']) ? $this->filesgroups['usertypes'][$filename]['admin'] : array(),
				'deny' => is_array($this->filesgroups['usertypes'][$filename]['deny']) ? $this->filesgroups['usertypes'][$filename]['deny'] : array()
				);
			$groups = array (
				'view' => is_array($this->filesgroups['groups'][$filename]['view']) ? $this->filesgroups['groups'][$filename]['view'] : array(),
				'admin' => is_array($this->filesgroups['groups'][$filename]['admin']) ? $this->filesgroups['groups'][$filename]['admin'] : array(),
				'deny' => is_array($this->filesgroups['groups'][$filename]['deny']) ? $this->filesgroups['groups'][$filename]['deny'] : array()
				);
			if ($perm == 'deny') continue;
			$title = ($title = $this->recordsByFilename[$filename]->title) ? $title : $entry;
			$description = $this->recordsByFilename[$filename]->description;
			$this->_entries[] = array(
				'basename' 		=> $entry,
				'filename' 		=> $filename,
				'realname' 		=> $realpath,
				'permission' 	=> $perm,
				'type' 			=> filetype($realname),
				'ext' 			=> $ext,
				'usertypes' 	=> $usertypes,
				'groups' 		=> $groups,
				'title' 		=> $title,
				'description' 	=> $description
			);
		}
		
		$d->close();
		
		# sort by directory
		usort($this->_entries, create_function ('$a, $b', 
			'return $a["type"] != $b["type"] 
						? ( $a["type"] == "dir" ? -1 : 1 )
						: strcmp($a["basename"], $b["basename"]);
			'));
			
		$this->_debug(&$this->_entries, '$this->_entries', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
			
		return true;
		
	}
	
	/**
	 * walk current sub dir/file
	 *
	 * @return boolean false if is no longer files
	 */
	function read() {
		if ($this->_entries_i < count($this->_entries)) {
			$this->entry = $this->_entries[$this->_entries_i++];
			return true;
		}
		$this->_entries_i = 0;
		return false;
	}
	
	
	function zipChildren($subs) {
		if (!$this->_recursed) $this->initDir(true);
		
		$this->_debug(&$subs, '$subs', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
		
		$zip = new ZipFile();
		foreach ($subs as $sub) 
			if ($sub = basename($sub))
				$this->_zipDir($zip, $sub);
		
		return $zip;
	}
	
	function _zipDir(&$zip, $dir) {
		if ( !$dir || 'deny' == $this->getPermission(VFM_Core::trimPath($this->dirname . '/' . $dir)) ) {
			$this->errorMsg = 'access denied';
			return false;
		}
	
		$realname = VFM_Core::realPath($this->conf['rootdir'], $this->dirname . '/' . $dir);
		$this->_debug(&$realname, '$realname', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
		
		# add file to zip
		if ( is_file($realname) ) {
			$this->_debug(&$dir, 'zip_file( $content, $dir )', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
			$zip->add_file (file_get_contents($realname), $dir);
		}
		
		# add dir and recursed to zip		
		elseif ( is_dir($realname) ) {
			$this->_debug(&$dir, 'zip_dir( $dir/ )', __LINE__, __FUNCTION__, __CLASS__, __FILE__);
			$zip->add_dir ($dir.'/');
			if ( $d = dir($realname) ) {
				while ($entry = $d->read())
					if ($entry != '.' && $entry != '..')
						$this->_zipDir ($zip, $dir.'/'.$entry);
			} else {
				$this->errorMsg = 'can not open directory';
				return false;
			}
		}
		
		# error...
		else {
			$this->errorMsg = 'does not exist';
			return false;
		}

		return true;
	}
	
	
	function download() {
		if ('deny' == $this->permission) {
			$this->errorMsg = 'access denied';
			return false;
		}
		
		$realname = $this->conf['rootdir'] . '/' . $this->dirname;
		
		if (!is_file($realname)) {
			$this->errorMsg = 'file is not found';
			return false;
		}
		
		return file_get_contents($realname);
	}
	
	
	function _debug($var = null, $varname = null, $line = null, $function, $class, $file = null, $backtrace = true) {
		if (!$this->conf['debug']) return true;
		ob_start();
		
		if (0 == $this->_debug_count++) {
			echo "<h1>DEBUG</h1>\n";
			echo "<p>" . date("Y-m-d H:i:s") . "</p>\n";
			echo "<hr />\n";
		}
		
		echo "<div onmouseover=\"this.style.backgroundColor='#ffffd3';\" onmouseout=\"this.style.backgroundColor='';\" onclick=\"if (this._expand) {this._expand=false; this.style.overflow='hidden'; this.style.height='20px';} else {this._expand=true; this.style.overflow='auto'; this.style.height='';}\" style=\"border:1px dashed; overflow:hidden; height: 20px; cursor: pointer\">\n";
		echo "<table cellpadding=\"0\" cellspacing=\"0\" width=\"100%\"><tr>\n";
		if ($line !== null) echo "<td nowrap width=\"15%\">LINE: <strong>".htmlspecialchars($line)."</strong></td>\n";
		if ($varname !== null) echo "<td nowrap width=\"35%\">VAR: <strong>".htmlspecialchars($varname)."</strong></td>\n";
		if ($function !== null) echo "<td nowrap width=\"35%\">FUNCTION: <strong>".htmlspecialchars($function)."</strong></td>\n";
		if ($class !== null) echo "<td nowrap width=\"15%\">CLASS: <strong>".htmlspecialchars($class)."</strong></td>\n";
		echo "</tr><tr>\n";
		if ($file !== null) echo "<td colspan=\"4\" nowrap>FILE: <strong>".htmlspecialchars($file)."</strong></td>\n";
		echo "</tr></table>\n";
		echo "<pre>\n";
		print_r($var); echo "\n";
		//debug_zval_dump($var);
		echo "</pre>\n";
		echo "<h2>BACKTRACK</h2>\n";
		echo "<pre style=\"background-color:#f0f0f0\">\n";
		if ($backtrace) debug_print_backtrace();
		echo "</pre>\n";
		echo "</div>\n";
		
		$contents = ob_get_contents();
		ob_end_clean();
		
		if ($this->conf['debug_output'])
			file_put_contents($this->conf['debug_output'], $contents, FILE_APPEND);
		else
			echo $contents;
	}
	
	function setDebugOutput($filename) {
		$this->conf['debug_output'] = $filename;
	}
}

?>